@@ -49,6 +49,26 @@ from picard.util import (
 )
 
 
+INVALID_CHARS = re.compile('([^\x20-\x7d]|=)')
+
+
+def sanitize_key(key):
+    """
+    Remove characters from key which are invalid for a Vorbis comment field name.
+    See https://www.xiph.org/vorbis/doc/v-comment.html#vectorformat
+    """
+    return INVALID_CHARS.sub('', key)
+
+
+def is_valid_key(key):
+    """
+    Return true if a string is a valid Vorbis comment key.
+    Valid characters for Vorbis comment field names are
+    ASCII 0x20 through 0x7D, 0x3D ('=') excluded.
+    """
+    return INVALID_CHARS.search(key) is None
+
+
 class VCommentFile(File):
 
     """Generic VComment-based file."""
@@ -94,7 +114,7 @@ class VCommentFile(File):
                         name, email = name.split(':', 1)
                     except ValueError:
                         email = ''
-                    if email != config.setting['rating_user_email']:
+                    if email != sanitize_key(config.setting['rating_user_email']):
                         continue
                     name = '~rating'
                     try:
@@ -113,8 +133,8 @@ class VCommentFile(File):
                         continue
                     name = "totaldiscs"
                 elif name == "metadata_block_picture":
-                    image = mutagen.flac.Picture(base64.standard_b64decode(value))
                     try:
+                        image = mutagen.flac.Picture(base64.standard_b64decode(value))
                         coverartimage = TagCoverArtImage(
                             file=filename,
                             tag=name,
@@ -123,11 +143,10 @@ class VCommentFile(File):
                             support_types=True,
                             data=image.data,
                         )
-                    except CoverArtImageError as e:
+                    except (CoverArtImageError, TypeError, ValueError, mutagen.flac.error) as e:
                         log.error('Cannot load image from %r: %s' % (filename, e))
                     else:
-                        metadata.append_image(coverartimage)
-
+                        metadata.images.append(coverartimage)
                     continue
                 elif name in self.__translate:
                     name = self.__translate[name]
@@ -146,7 +165,7 @@ class VCommentFile(File):
                 except CoverArtImageError as e:
                     log.error('Cannot load image from %r: %s' % (filename, e))
                 else:
-                    metadata.append_image(coverartimage)
+                    metadata.images.append(coverartimage)
 
         # Read the unofficial COVERART tags, for backward compatibility only
         if "metadata_block_picture" not in file.tags:
@@ -158,10 +177,10 @@ class VCommentFile(File):
                             tag='COVERART',
                             data=base64.standard_b64decode(data)
                         )
-                    except CoverArtImageError as e:
+                    except (CoverArtImageError, TypeError, ValueError) as e:
                         log.error('Cannot load image from %r: %s' % (filename, e))
                     else:
-                        metadata.append_image(coverartimage)
+                        metadata.images.append(coverartimage)
             except KeyError:
                 pass
         self._info(metadata, file)
@@ -179,15 +198,16 @@ class VCommentFile(File):
             file.tags.clear()
             if channel_mask:
                 file.tags['waveformatextensible_channel_mask'] = channel_mask
-        if (is_flac and (config.setting["clear_existing_tags"] or
-                         metadata.images_to_be_saved_to_tags)):
+        images_to_save = list(metadata.images.to_be_saved_to_tags())
+        if is_flac and (config.setting["clear_existing_tags"] or images_to_save):
             file.clear_pictures()
         tags = {}
         for name, value in metadata.items():
             if name == '~rating':
                 # Save rating according to http://code.google.com/p/quodlibet/wiki/Specs_VorbisComments
-                if config.setting['rating_user_email']:
-                    name = 'rating:%s' % config.setting['rating_user_email']
+                user_email = sanitize_key(config.setting['rating_user_email'])
+                if user_email:
+                    name = 'rating:%s' % user_email
                 else:
                     name = 'rating'
                 value = str(float(value) / (config.setting['rating_steps'] - 1))
@@ -216,7 +236,7 @@ class VCommentFile(File):
         if "totaldiscs" in metadata:
             tags.setdefault("DISCTOTAL", []).append(metadata["totaldiscs"])
 
-        for image in metadata.images_to_be_saved_to_tags:
+        for image in images_to_save:
             picture = mutagen.flac.Picture()
             picture.data = image.data
             picture.mime = image.mimetype
@@ -283,8 +303,9 @@ class VCommentFile(File):
 
     @classmethod
     def supports_tag(cls, name):
-        unsupported_tags = {}
-        return bool(name) and name not in unsupported_tags
+        unsupported_tags = ['r128_album_gain', 'r128_track_gain']
+        return (bool(name) and name not in unsupported_tags
+                and is_valid_key(name))
 
 
 class FLACFile(VCommentFile):
@@ -294,10 +315,6 @@ class FLACFile(VCommentFile):
     NAME = "FLAC"
     _File = mutagen.flac.FLAC
 
-    def _info(self, metadata, file):
-        super()._info(metadata, file)
-        metadata['~format'] = self.NAME
-
 
 class OggFLACFile(VCommentFile):
 
@@ -322,6 +339,10 @@ class OggTheoraFile(VCommentFile):
     NAME = "Ogg Theora"
     _File = mutagen.oggtheora.OggTheora
 
+    def _info(self, metadata, file):
+        super()._info(metadata, file)
+        metadata['~video'] = '1'
+
 
 class OggVorbisFile(VCommentFile):
 
@@ -338,6 +359,12 @@ class OggOpusFile(VCommentFile):
     NAME = "Ogg Opus"
     _File = mutagen.oggopus.OggOpus
 
+    @classmethod
+    def supports_tag(cls, name):
+        if name.startswith('r128_'):
+            return True
+        return VCommentFile.supports_tag(name)
+
 
 def OggAudioFile(filename):
     """Generic Ogg audio file."""
@@ -347,6 +374,7 @@ def OggAudioFile(filename):
 
 OggAudioFile.EXTENSIONS = [".oga"]
 OggAudioFile.NAME = "Ogg Audio"
+OggAudioFile.supports_tag = VCommentFile.supports_tag
 
 
 def OggVideoFile(filename):
@@ -357,3 +385,4 @@ def OggVideoFile(filename):
 
 OggVideoFile.EXTENSIONS = [".ogv"]
 OggVideoFile.NAME = "Ogg Video"
+OggVideoFile.supports_tag = VCommentFile.supports_tag
